@@ -1,32 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "glslengine.h"
 #include "glslsymbols.h"
@@ -92,30 +65,30 @@ Engine::~Engine()
 
 const QString *Engine::identifier(const QString &s)
 {
-    return &(*_identifiers.insert(s));
+    return &(*_identifiers.insert(s).first);
 }
 
 const QString *Engine::identifier(const char *s, int n)
 {
-    return &(*_identifiers.insert(QString::fromLatin1(s, n)));
+    return &(*_identifiers.insert(QString::fromLatin1(s, n)).first);
 }
 
-QSet<QString> Engine::identifiers() const
+std::unordered_set<QString> Engine::identifiers() const
 {
     return _identifiers;
 }
 
 const QString *Engine::number(const QString &s)
 {
-    return &(*_numbers.insert(s));
+    return &(*_numbers.insert(s).first);
 }
 
 const QString *Engine::number(const char *s, int n)
 {
-    return &(*_numbers.insert(QString::fromLatin1(s, n)));
+    return &(*_numbers.insert(QString::fromLatin1(s, n)).first);
 }
 
-QSet<QString> Engine::numbers() const
+std::unordered_set<QString> Engine::numbers() const
 {
     return _numbers;
 }
@@ -172,6 +145,11 @@ const SamplerType *Engine::samplerType(int kind)
     return _samplerTypes.intern(SamplerType(kind));
 }
 
+const ImageType *Engine::imageType(int kind)
+{
+    return _imageTypes.intern(ImageType(kind));
+}
+
 const VectorType *Engine::vectorType(const Type *elementType, int dimension)
 {
     VectorType *type = const_cast<VectorType *>
@@ -191,7 +169,6 @@ const ArrayType *Engine::arrayType(const Type *elementType)
     return _arrayTypes.intern(ArrayType(elementType));
 }
 
-
 QList<DiagnosticMessage> Engine::diagnosticMessages() const
 {
     return _diagnosticMessages;
@@ -264,6 +241,13 @@ Function *Engine::newFunction(Scope *scope)
     return s;
 }
 
+InterfaceBlock *Engine::newInterfaceBlock(Scope *scope)
+{
+    InterfaceBlock *iBlock = new InterfaceBlock(scope);
+    _symbols.append(iBlock);
+    return iBlock;
+}
+
 Argument *Engine::newArgument(Function *function, const QString &name, const Type *type)
 {
     Argument *a = new Argument(function);
