@@ -1,32 +1,11 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Jochen Becher
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt Creator.
-**
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-****************************************************************************/
+// Copyright (C) 2016 Jochen Becher
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "defaultstyleengine.h"
 
 #include "defaultstyle.h"
 #include "objectvisuals.h"
+#include "relationvisuals.h"
 #include "styledobject.h"
 #include "styledrelation.h"
 
@@ -36,6 +15,7 @@
 #include "qmt/diagram/ditem.h"
 #include "qmt/diagram/dannotation.h"
 #include "qmt/infrastructure/qmtassert.h"
+#include "qmt/stereotype/customrelation.h"
 
 #include <utils/algorithm.h>
 
@@ -82,7 +62,7 @@ public:
     ObjectVisuals m_objectVisuals;
 };
 
-uint qHash(const ObjectStyleKey &styleKey)
+size_t qHash(const ObjectStyleKey &styleKey)
 {
     return ::qHash(styleKey.m_elementType) ^ qHash(styleKey.m_objectVisuals);
 }
@@ -95,25 +75,29 @@ bool operator==(const ObjectStyleKey &lhs, const ObjectStyleKey &rhs)
 class RelationStyleKey
 {
 public:
-    RelationStyleKey(StyleEngine::ElementType elementType = StyleEngine::TypeOther,
-                     DObject::VisualPrimaryRole visualPrimaryRole = DObject::PrimaryRoleNormal)
+    RelationStyleKey(StyleEngine::ElementType elementType, const RelationVisuals &relationVisuals,
+                     bool withObject = false)
         : m_elementType(elementType),
-          m_visualPrimaryRole(visualPrimaryRole)
+          m_relationVisuals(relationVisuals),
+        m_withObject(withObject)
     {
     }
 
     StyleEngine::ElementType m_elementType = StyleEngine::TypeOther;
-    DObject::VisualPrimaryRole m_visualPrimaryRole = DObject::PrimaryRoleNormal;
+    RelationVisuals m_relationVisuals;
+    bool m_withObject = false;
 };
 
-uint qHash(const RelationStyleKey &styleKey)
+size_t qHash(const RelationStyleKey &styleKey)
 {
-    return ::qHash(styleKey.m_elementType) ^ ::qHash(styleKey.m_visualPrimaryRole);
+    return ::qHash(styleKey.m_elementType) ^ qHash(styleKey.m_relationVisuals);
 }
 
 bool operator==(const RelationStyleKey &lhs, const RelationStyleKey &rhs)
 {
-    return lhs.m_elementType == rhs.m_elementType && lhs.m_visualPrimaryRole == rhs.m_visualPrimaryRole;
+    return lhs.m_elementType == rhs.m_elementType
+           && lhs.m_relationVisuals == rhs.m_relationVisuals
+           && lhs.m_withObject == rhs.m_withObject;
 }
 
 class AnnotationStyleKey
@@ -127,7 +111,7 @@ public:
     DAnnotation::VisualRole m_visualRole = DAnnotation::RoleNormal;
 };
 
-uint qHash(const AnnotationStyleKey &styleKey)
+size_t qHash(const AnnotationStyleKey &styleKey)
 {
     return ::qHash(styleKey.m_visualRole);
 }
@@ -142,17 +126,17 @@ class BoundaryStyleKey
 {
 };
 
-uint qHash(const BoundaryStyleKey &styleKey)
+size_t qHash(const BoundaryStyleKey &styleKey)
 {
-    Q_UNUSED(styleKey);
+    Q_UNUSED(styleKey)
 
-    return 1;
+    return ::qHash(1);
 }
 
 bool operator==(const BoundaryStyleKey &lhs, const BoundaryStyleKey &rhs)
 {
-    Q_UNUSED(lhs);
-    Q_UNUSED(rhs);
+    Q_UNUSED(lhs)
+    Q_UNUSED(rhs)
 
     return true;
 }
@@ -162,17 +146,17 @@ class SwimlaneStyleKey
 {
 };
 
-uint qHash(const SwimlaneStyleKey &styleKey)
+size_t qHash(const SwimlaneStyleKey &styleKey)
 {
-    Q_UNUSED(styleKey);
+    Q_UNUSED(styleKey)
 
-    return 1;
+    return ::qHash(1);
 }
 
 bool operator==(const SwimlaneStyleKey &lhs, const SwimlaneStyleKey &rhs)
 {
-    Q_UNUSED(lhs);
-    Q_UNUSED(rhs);
+    Q_UNUSED(lhs)
+    Q_UNUSED(rhs)
 
     return true;
 }
@@ -284,7 +268,8 @@ const Style *DefaultStyleEngine::applyObjectStyle(const Style *baseStyle, const
     DObject::VisualPrimaryRole styledVisualPrimaryRole = styledObject.objectVisuals().visualPrimaryRole();
     DObject::VisualSecondaryRole styledVisualSecondaryRole = styledObject.objectVisuals().visualSecondaryRole();
     QHash<int, DepthProperties> depths;
-    foreach (const DObject *collidingObject, styledObject.collidingObjects()) {
+    const QList<const DObject *> collidingObjectList = styledObject.collidingObjects();
+    for (const DObject *collidingObject : collidingObjectList) {
         int collidingDepth = collidingObject->depth();
         if (collidingDepth < styledObject.object()->depth()) {
             ElementType collidingElementType = objectType(collidingObject);
@@ -318,9 +303,8 @@ const Style *DefaultStyleEngine::applyObjectStyle(const Style *baseStyle, const
     }
     int depth = 0;
     if (!depths.isEmpty()) {
-        QList<int> keys = depths.keys();
-        Utils::sort(keys);
-        foreach (int d, keys) {
+        const QList<int> keys = Utils::sorted(depths.keys());
+        for (int d : keys) {
             DepthProperties properties = depths.value(d);
             if (properties.m_elementType == elementType
                     && areStackingRoles(properties.m_visualPrimaryRole, properties.m_visualSecondaryRole,
@@ -341,13 +325,100 @@ const Style *DefaultStyleEngine::applyObjectStyle(const Style *baseStyle, const
                             parameters);
 }
 
+const Style *DefaultStyleEngine::applyRelationStyle(const Style *baseStyle, ElementType elementType, const RelationVisuals &relationVisuals, const Parameters *parameters)
+{
+    Q_UNUSED(parameters)
+
+    RelationStyleKey key(elementType, relationVisuals);
+    const Style *derivedStyle = m_relationStyleMap.value(key);
+    if (!derivedStyle) {
+        auto style = new Style(baseStyle->type());
+        static QColor customColors[] = {
+            QColor(0xEE, 0x8E, 0x99).darker(110),  // ROLE_CUSTOM1,
+            QColor(0x80, 0xAF, 0x47).lighter(130), // ROLE_CUSTOM2,
+            QColor(0xFF, 0xA1, 0x5B).lighter(100), // ROLE_CUSTOM3,
+            QColor(0x55, 0xC4, 0xCF).lighter(120), // ROLE_CUSTOM4,
+            QColor(0xFF, 0xE1, 0x4B)               // ROLE_CUSTOM5,
+        };
+
+        int index = static_cast<int>(relationVisuals.visualPrimaryRole()) - static_cast<int>(DRelation::PrimaryRoleCustom1);
+        QColor lineColor = index >= 0 && index <= 4 ? customColors[index] : Qt::black;
+        switch (relationVisuals.visualSecondaryRole()) {
+        case DRelation::SecondaryRoleNone:
+            break;
+        case DRelation::SecondaryRoleWarning:
+            lineColor = Qt::yellow;
+            break;
+        case DRelation::SecondaryRoleError:
+            lineColor = Qt::red;
+            break;
+        case DRelation::SecondaryRoleSoften:
+            lineColor = Qt::gray;
+            break;
+        }
+
+        QColor fillColor = lineColor == Qt::black ? Qt::darkGray : lineColor.lighter(150);
+        QPen linePen = baseStyle->linePen();
+        linePen.setWidth(1);
+        linePen.setColor(lineColor);
+        style->setLinePen(linePen);
+        QBrush textBrush = baseStyle->textBrush();
+        textBrush.setColor(Qt::black);
+        style->setTextBrush(textBrush);
+        QBrush brush = baseStyle->fillBrush();
+        brush.setColor(fillColor);
+        brush.setStyle(Qt::SolidPattern);
+        style->setFillBrush(brush);
+        style->setNormalFont(baseStyle->normalFont());
+        style->setSmallFont(baseStyle->smallFont());
+        style->setHeaderFont(baseStyle->headerFont());
+        m_relationStyleMap.insert(key, style);
+        derivedStyle = style;
+    }
+    return derivedStyle;
+}
+
 const Style *DefaultStyleEngine::applyRelationStyle(const Style *baseStyle, const StyledRelation &styledRelation,
                                                     const Parameters *parameters)
 {
-    Q_UNUSED(parameters);
+    Q_UNUSED(parameters)
 
     ElementType elementType = objectType(styledRelation.endA());
-    RelationStyleKey key(elementType, styledRelation.endA() ? styledRelation.endA()->visualPrimaryRole() : DObject::PrimaryRoleNormal);
+    RelationVisuals relationVisuals;
+    if (styledRelation.customRelation()) {
+        switch (styledRelation.customRelation()->colorType()) {
+        case CustomRelation::ColorType::EndA:
+            // TODO implement
+            break;
+        case CustomRelation::ColorType::EndB:
+            // TODO implement
+            break;
+        case CustomRelation::ColorType::Custom:
+            // TODO implement
+            break;
+        case CustomRelation::ColorType::Warning:
+            relationVisuals.setVisualSecondaryRole(DRelation::VisualSecondaryRole::SecondaryRoleWarning);
+            break;
+        case CustomRelation::ColorType::Error:
+            relationVisuals.setVisualSecondaryRole(DRelation::VisualSecondaryRole::SecondaryRoleError);
+            break;
+        case CustomRelation::ColorType::Soften:
+            relationVisuals.setVisualSecondaryRole(DRelation::VisualSecondaryRole::SecondaryRoleSoften);
+            break;
+        }
+        relationVisuals.setEmphasized(styledRelation.customRelation()->emphasized());
+    }
+    if (styledRelation.endA())
+        relationVisuals.setVisualObjectPrimaryRole(styledRelation.endA()->visualPrimaryRole());
+    if (styledRelation.relation()) {
+        if (styledRelation.relation()->visualPrimaryRole() != DRelation::VisualPrimaryRole::PrimaryRoleNormal)
+            relationVisuals.setVisualPrimaryRole(styledRelation.relation()->visualPrimaryRole());
+        if (styledRelation.relation()->visualSecondaryRole() != DRelation::VisualSecondaryRole::SecondaryRoleNone)
+            relationVisuals.setVisualSecondaryRole(styledRelation.relation()->visualSecondaryRole());
+        if (styledRelation.relation()->isVisualEmphasized())
+            relationVisuals.setEmphasized(styledRelation.relation()->isVisualEmphasized());
+    }
+    RelationStyleKey key(elementType, relationVisuals, true);
     const Style *derivedStyle = m_relationStyleMap.value(key);
     if (!derivedStyle) {
         auto style = new Style(baseStyle->type());
@@ -361,12 +432,36 @@ const Style *DefaultStyleEngine::applyRelationStyle(const Style *baseStyle, cons
         QColor lineColor = DefaultStyleEngine::lineColor(objectType(object), objectVisuals);
         QColor fillColor = lineColor;
 
+        static QColor customColors[] = {
+            QColor(0xEE, 0x8E, 0x99).darker(110),  // ROLE_CUSTOM1,
+            QColor(0x80, 0xAF, 0x47).lighter(130), // ROLE_CUSTOM2,
+            QColor(0xFF, 0xA1, 0x5B).lighter(100), // ROLE_CUSTOM3,
+            QColor(0x55, 0xC4, 0xCF).lighter(120), // ROLE_CUSTOM4,
+            QColor(0xFF, 0xE1, 0x4B)               // ROLE_CUSTOM5,
+        };
+
+        int index = static_cast<int>(relationVisuals.visualPrimaryRole()) - static_cast<int>(DRelation::PrimaryRoleCustom1);
+        lineColor = (index >= 0 && index <= 4) ? customColors[index] : lineColor;
+        switch (relationVisuals.visualSecondaryRole()) {
+        case DRelation::SecondaryRoleNone:
+            break;
+        case DRelation::SecondaryRoleWarning:
+            lineColor = QColor(0xffc800);
+            break;
+        case DRelation::SecondaryRoleError:
+            lineColor = Qt::red;
+            break;
+        case DRelation::SecondaryRoleSoften:
+            lineColor = Qt::gray;
+            break;
+        }
+
         QPen linePen = baseStyle->linePen();
-        linePen.setWidth(1);
+        linePen.setWidth(relationVisuals.isEmphasized() ? 3 : 1);
         linePen.setColor(lineColor);
         style->setLinePen(linePen);
         QBrush textBrush = baseStyle->textBrush();
-        textBrush.setColor(QColor("black"));
+        textBrush.setColor(Qt::black);
         style->setTextBrush(textBrush);
         QBrush brush = baseStyle->fillBrush();
         brush.setColor(fillColor);
@@ -391,14 +486,14 @@ const Style *DefaultStyleEngine::applyAnnotationStyle(const Style *baseStyle, co
 const Style *DefaultStyleEngine::applyBoundaryStyle(const Style *baseStyle, const DBoundary *boundary,
                                                     const Parameters *parameters)
 {
-    Q_UNUSED(boundary);
+    Q_UNUSED(boundary)
 
     return applyBoundaryStyle(baseStyle, parameters);
 }
 
 const Style *DefaultStyleEngine::applySwimlaneStyle(const Style *baseStyle, const DSwimlane *swimlane, const StyleEngine::Parameters *parameters)
 {
-    Q_UNUSED(swimlane);
+    Q_UNUSED(swimlane)
 
     return applySwimlaneStyle(baseStyle, parameters);
 }
@@ -406,7 +501,7 @@ const Style *DefaultStyleEngine::applySwimlaneStyle(const Style *baseStyle, cons
 const Style *DefaultStyleEngine::applyAnnotationStyle(const Style *baseStyle, DAnnotation::VisualRole visualRole,
                                                       const StyleEngine::Parameters *parameters)
 {
-    Q_UNUSED(parameters);
+    Q_UNUSED(parameters)
 
     AnnotationStyleKey key(visualRole);
     const Style *derivedStyle = m_annotationStyleMap.value(key);
@@ -447,7 +542,7 @@ const Style *DefaultStyleEngine::applyAnnotationStyle(const Style *baseStyle, DA
 
 const Style *DefaultStyleEngine::applyBoundaryStyle(const Style *baseStyle, const StyleEngine::Parameters *parameters)
 {
-    Q_UNUSED(parameters);
+    Q_UNUSED(parameters)
 
     BoundaryStyleKey key;
     const Style *derivedStyle = m_boundaryStyleMap.value(key);
@@ -463,7 +558,7 @@ const Style *DefaultStyleEngine::applyBoundaryStyle(const Style *baseStyle, cons
 
 const Style *DefaultStyleEngine::applySwimlaneStyle(const Style *baseStyle, const StyleEngine::Parameters *parameters)
 {
-    Q_UNUSED(parameters);
+    Q_UNUSED(parameters)
 
     SwimlaneStyleKey key;
     const Style *derivedStyle = m_swimlaneStyleMap.value(key);
@@ -524,7 +619,7 @@ bool DefaultStyleEngine::areStackingRoles(DObject::VisualPrimaryRole rhsPrimaryR
 QColor DefaultStyleEngine::baseColor(ElementType elementType, ObjectVisuals objectVisuals)
 {
     if (objectVisuals.visualSecondaryRole() == DObject::SecondaryRoleOutline)
-        return QColor("#FFFFFF");
+        return QColor(0xFF, 0xFF, 0xFF);
 
     QColor baseColor;
 
@@ -534,33 +629,33 @@ QColor DefaultStyleEngine::baseColor(ElementType elementType, ObjectVisuals obje
         } else {
             switch (elementType) {
             case TypePackage:
-                baseColor = QColor("#7C98AD");
+                baseColor = QColor(0x7C, 0x98, 0xAD);
                 break;
             case TypeComponent:
-                baseColor = QColor("#A0A891");
+                baseColor = QColor(0xA0, 0xA8, 0x91);
                 break;
             case TypeClass:
-                baseColor = QColor("#E5A858");
+                baseColor = QColor(0xE5, 0xA8, 0x58);
                 break;
             case TypeItem:
-                baseColor = QColor("#B995C6");
+                baseColor = QColor(0xB9, 0x95, 0xC6);
                 break;
             case TypeRelation:
             case TypeAnnotation:
             case TypeBoundary:
             case TypeSwimlane:
             case TypeOther:
-                baseColor = QColor("#BF7D65");
+                baseColor = QColor(0xBF, 0x7D, 0x65);
                 break;
             }
         }
     } else {
         static QColor customColors[] = {
-            QColor("#EE8E99").darker(110),  // ROLE_CUSTOM1,
-            QColor("#80AF47").lighter(130), // ROLE_CUSTOM2,
-            QColor("#FFA15B").lighter(100), // ROLE_CUSTOM3,
-            QColor("#55C4CF").lighter(120), // ROLE_CUSTOM4,
-            QColor("#FFE14B")               // ROLE_CUSTOM5,
+            QColor(0xEE, 0x8E, 0x99).darker(110),  // ROLE_CUSTOM1,
+            QColor(0x80, 0xAF, 0x47).lighter(130), // ROLE_CUSTOM2,
+            QColor(0xFF, 0xA1, 0x5B).lighter(100), // ROLE_CUSTOM3,
+            QColor(0x55, 0xC4, 0xCF).lighter(120), // ROLE_CUSTOM4,
+            QColor(0xFF, 0xE1, 0x4B)               // ROLE_CUSTOM5,
         };
 
         int index = static_cast<int>(objectVisuals.visualPrimaryRole()) - static_cast<int>(DObject::PrimaryRoleCustom1);
@@ -614,7 +709,7 @@ QColor DefaultStyleEngine::fillColor(ElementType elementType, const ObjectVisual
 
 QColor DefaultStyleEngine::textColor(const DObject *object, int depth)
 {
-    Q_UNUSED(depth);
+    Q_UNUSED(depth)
 
     QColor textColor;
     DObject::VisualPrimaryRole visualRole = object ? object->visualPrimaryRole() : DObject::PrimaryRoleNormal;
@@ -627,7 +722,7 @@ QColor DefaultStyleEngine::textColor(const DObject *object, int depth)
 
 QColor DefaultStyleEngine::textColor(ElementType elementType, const ObjectVisuals &objectVisuals)
 {
-    Q_UNUSED(elementType);
+    Q_UNUSED(elementType)
 
     QColor textColor;
     if (objectVisuals.visualSecondaryRole() == DObject::SecondaryRoleSoften)
