@@ -24,19 +24,21 @@
 #include <projectexplorer/buildmanager.h>
 #include <projectexplorer/buildpropertiessettings.h>
 #include <projectexplorer/buildsteplist.h>
+#include <projectexplorer/devicesupport/devicekitaspects.h>
 #include <projectexplorer/kit.h>
 #include <projectexplorer/makestep.h>
 #include <projectexplorer/projectexplorerconstants.h>
-#include <projectexplorer/projectexplorertr.h>
 #include <projectexplorer/runconfiguration.h>
+#include <projectexplorer/sysrootkitaspect.h>
 #include <projectexplorer/target.h>
 #include <projectexplorer/toolchain.h>
+#include <projectexplorer/toolchainkitaspect.h>
 
 #include <qtsupport/qtbuildaspects.h>
 #include <qtsupport/qtkitaspect.h>
 #include <qtsupport/qtversionmanager.h>
 
-#include <utils/process.h>
+#include <utils/qtcprocess.h>
 #include <utils/mimeconstants.h>
 #include <utils/qtcassert.h>
 
@@ -73,7 +75,7 @@ FilePath QmakeBuildConfiguration::shadowBuildDirectory(const FilePath &proFilePa
         return {};
 
     const QString projectName = proFilePath.completeBaseName();
-    return buildDirectoryFromTemplate(Project::projectDirectory(proFilePath), proFilePath,
+    return buildDirectoryFromTemplate(proFilePath.absolutePath(), proFilePath,
                                       projectName, k, suffix, buildType, "qmake");
 }
 
@@ -85,18 +87,16 @@ QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target, Id id)
     setConfigWidgetDisplayName(Tr::tr("General"));
     setConfigWidgetHasFrame(true);
 
-    m_buildSystem = new QmakeBuildSystem(this);
-
     appendInitialBuildStep(Constants::QMAKE_BS_ID);
     appendInitialBuildStep(Constants::MAKESTEP_BS_ID);
     appendInitialCleanStep(Constants::MAKESTEP_BS_ID);
 
-    setInitializer([this, target](const BuildInfo &info) {
+    setInitializer([this](const BuildInfo &info) {
         QMakeStep *qmakeStep = buildSteps()->firstOfType<QMakeStep>();
         QTC_ASSERT(qmakeStep, return);
 
         const QmakeExtraBuildInfo qmakeExtra = info.extraInfo.value<QmakeExtraBuildInfo>();
-        QtVersion *version = QtKitAspect::qtVersion(target->kit());
+        QtVersion *version = QtKitAspect::qtVersion(kit());
 
         QtVersion::QmakeBuildConfigs config = version->defaultBuildConfig();
         if (info.buildType == BuildConfiguration::Debug)
@@ -116,14 +116,14 @@ QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target, Id id)
 
         FilePath directory = info.buildDirectory;
         if (directory.isEmpty()) {
-            directory = shadowBuildDirectory(target->project()->projectFilePath(),
-                                             target->kit(), info.displayName,
+            directory = shadowBuildDirectory(project()->projectFilePath(),
+                                             kit(), info.displayName,
                                              info.buildType);
         }
 
         setBuildDirectory(directory);
 
-        if (DeviceTypeKitAspect::deviceTypeId(target->kit())
+        if (RunDeviceTypeKitAspect::deviceTypeId(kit())
                         == Android::Constants::ANDROID_DEVICE_TYPE) {
             buildSteps()->appendStep(Android::Constants::ANDROID_PACKAGE_INSTALL_STEP_ID);
             buildSteps()->appendStep(Android::Constants::ANDROID_BUILD_APK_ID);
@@ -132,8 +132,7 @@ QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target, Id id)
         updateCacheAndEmitEnvironmentChanged();
     });
 
-    connect(target, &Target::kitChanged,
-            this, &QmakeBuildConfiguration::kitChanged);
+    connect(this, &BuildConfiguration::kitChanged, this, &QmakeBuildConfiguration::kitChanged);
     MacroExpander *expander = macroExpander();
     expander->registerVariable("Qmake:Makefile", "Qmake makefile", [this]() -> QString {
         const FilePath file = makefile();
@@ -142,7 +141,7 @@ QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target, Id id)
         return QLatin1String("Makefile");
     });
 
-    buildDirectoryAspect()->allowInSourceBuilds(target->project()->projectDirectory());
+    buildDirectoryAspect()->allowInSourceBuilds(project()->projectDirectory());
     connect(this, &BuildConfiguration::buildDirectoryInitialized,
             this, &QmakeBuildConfiguration::updateProblemLabel);
     connect(this, &BuildConfiguration::buildDirectoryChanged,
@@ -151,8 +150,10 @@ QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target, Id id)
             this, &QmakeBuildConfiguration::updateProblemLabel);
     connect(&settings(), &AspectContainer::changed,
             this, &QmakeBuildConfiguration::updateProblemLabel);
-    connect(target, &Target::parsingFinished, this, &QmakeBuildConfiguration::updateProblemLabel);
-    connect(target, &Target::kitChanged, this, &QmakeBuildConfiguration::updateProblemLabel);
+    m_bsParsingFinishedConnection = connect(buildSystem(), &BuildSystem::parsingFinished,
+            this, &QmakeBuildConfiguration::updateProblemLabel);
+    connect(this, &BuildConfiguration::kitChanged,
+            this, &QmakeBuildConfiguration::updateProblemLabel);
 
     connect(&separateDebugInfo, &BaseAspect::changed, this, [this] {
         emit separateDebugInfoChanged();
@@ -160,14 +161,12 @@ QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target, Id id)
         qmakeBuildSystem()->scheduleUpdateAllNowOrLater();
     });
 
-    qmlDebugging.setBuildConfiguration(this);
     connect(&qmlDebugging, &BaseAspect::changed, this, [this] {
         emit qmlDebuggingChanged();
         emit qmakeBuildConfigurationChanged();
         qmakeBuildSystem()->scheduleUpdateAllNowOrLater();
     });
 
-    useQtQuickCompiler.setBuildConfiguration(this);
     connect(&useQtQuickCompiler, &QtQuickCompilerAspect::changed, this, [this] {
         emit useQtQuickCompilerChanged();
         emit qmakeBuildConfigurationChanged();
@@ -185,7 +184,9 @@ QmakeBuildConfiguration::QmakeBuildConfiguration(Target *target, Id id)
 
 QmakeBuildConfiguration::~QmakeBuildConfiguration()
 {
-    delete m_buildSystem;
+    // BuildSystem might send signals during destruction before this QObject
+    // had a chance to disconnect from it.
+    disconnect(m_bsParsingFinishedConnection);
 }
 
 void QmakeBuildConfiguration::toMap(Store &map) const
@@ -212,7 +213,7 @@ void QmakeBuildConfiguration::kitChanged()
         // This only checks if the ids have changed!
         // For that reason the QmakeBuildConfiguration is also connected
         // to the toolchain and qtversion managers
-        m_buildSystem->scheduleUpdateAllNowOrLater();
+        qmakeBuildSystem()->scheduleUpdateAllNowOrLater();
         m_lastKitState = newState;
     }
 }
@@ -240,6 +241,7 @@ void QmakeBuildConfiguration::updateProblemLabel()
     bool targetMismatch = false;
     bool incompatibleBuild = false;
     bool allGood = false;
+    bool invalidArguments = false;
     // we only show if we actually have a qmake and makestep
     QString errorString;
     if (qmakeStep() && makeStep()) {
@@ -257,6 +259,9 @@ void QmakeBuildConfiguration::updateProblemLabel()
         case QmakeBuildConfiguration::MakefileForWrongProject:
             targetMismatch = true;
             break;
+        case QmakeBuildConfiguration::InvalidArguments:
+            invalidArguments = true;
+            break;
         }
     }
 
@@ -272,7 +277,7 @@ void QmakeBuildConfiguration::updateProblemLabel()
             QString text = QLatin1String("<nobr>");
             for (const ProjectExplorer::Task &task : issues) {
                 QString type;
-                switch (task.type) {
+                switch (task.type()) {
                 case ProjectExplorer::Task::Error:
                     type = Tr::tr("Error:");
                     type += QLatin1Char(' ');
@@ -304,16 +309,15 @@ void QmakeBuildConfiguration::updateProblemLabel()
     } else if (unalignedBuildDir) {
         buildDirectoryAspect()->setProblem(unalignedBuildDirWarning());
         return;
+    } else if (invalidArguments) {
+        buildDirectoryAspect()->setProblem(
+            Tr::tr("Starting qmake failed with the following error: %1").arg(errorString));
+        return;
     }
 
     buildDirectoryAspect()->setProblem({});
 }
 
-BuildSystem *QmakeBuildConfiguration::buildSystem() const
-{
-    return m_buildSystem;
-}
-
 /// If only a sub tree should be build this function returns which sub node
 /// should be build
 /// \see QMakeBuildConfiguration::setSubNodeBuild
@@ -345,7 +349,7 @@ void QmakeBuildConfiguration::setFileNodeBuild(FileNode *node)
 
 FilePath QmakeBuildConfiguration::makefile() const
 {
-    return FilePath::fromString(m_buildSystem->rootProFile()->singleVariableValue(Variable::Makefile));
+    return FilePath::fromString(qmakeBuildSystem()->rootProFile()->singleVariableValue(Variable::Makefile));
 }
 
 QtVersion::QmakeBuildConfigs QmakeBuildConfiguration::qmakeBuildConfiguration() const
@@ -360,7 +364,7 @@ void QmakeBuildConfiguration::setQMakeBuildConfiguration(QtVersion::QmakeBuildCo
     m_qmakeBuildConfiguration = config;
 
     emit qmakeBuildConfigurationChanged();
-    m_buildSystem->scheduleUpdateAllNowOrLater();
+    qmakeBuildSystem()->scheduleUpdateAllNowOrLater();
     emit buildTypeChanged();
 }
 
@@ -406,6 +410,25 @@ bool QmakeBuildConfiguration::runQmakeSystemFunctions() const
     return settings().runSystemFunction();
 }
 
+void QmakeBuildConfiguration::setInitialArgs(const QStringList &args)
+{
+    if (BuildStepList *buildSteps = this->buildSteps()) {
+        if (auto qmakeStep = buildSteps->firstOfType<QmakeProjectManager::QMakeStep>())
+            qmakeStep->userArguments.setArguments(ProcessArgs::joinArgs(args));
+    }
+}
+
+QStringList QmakeBuildConfiguration::initialArgs() const
+{
+    if (BuildStepList *buildSteps = this->buildSteps()) {
+        if (auto qmakeStep = buildSteps->firstOfType<QmakeProjectManager::QMakeStep>()) {
+            return ProcessArgs::splitArgs(qmakeStep->userArguments.arguments(),
+                                          project()->projectFilePath().osType());
+        }
+    }
+    return {};
+}
+
 QStringList QmakeBuildConfiguration::configCommandLineArguments() const
 {
     QStringList result;
@@ -447,7 +470,7 @@ MakeStep *QmakeBuildConfiguration::makeStep() const
 
 QmakeBuildSystem *QmakeBuildConfiguration::qmakeBuildSystem() const
 {
-    return m_buildSystem;
+    return qobject_cast<QmakeBuildSystem *>(buildSystem());
 }
 
 // Returns true if both are equal.
@@ -515,8 +538,16 @@ QmakeBuildConfiguration::MakefileState QmakeBuildConfiguration::compareToImportF
     // and compare that on its own
     FilePath workingDirectory = makefile.parentDir();
     QStringList actualArgs;
-    QString allArgs = macroExpander()->expandProcessArgs(qs->allArguments(
-        QtKitAspect::qtVersion(target()->kit()), QMakeStep::ArgumentFlag::Expand));
+    Result<QString> expandResult = macroExpander()->expandProcessArgs(
+        qs->allArguments(QtKitAspect::qtVersion(kit()), QMakeStep::ArgumentFlag::Expand));
+
+    if (!expandResult) {
+        if (errorString)
+            *errorString = expandResult.error();
+        return InvalidArguments;
+    }
+
+    QString allArgs = *expandResult;
     // This copies the settings from allArgs to actualArgs (minus some we
     // are not interested in), splitting them up into individual strings:
     extractSpecFromArguments(&allArgs, workingDirectory, version, &actualArgs);
@@ -647,7 +678,7 @@ QString QmakeBuildConfiguration::extractSpecFromArguments(QString *args,
         if (parsedSpec.isChildOf(sourceMkSpecPath))
             parsedSpec = parsedSpec.relativeChildPath(sourceMkSpecPath);
     }
-    return parsedSpec.toString();
+    return parsedSpec.toUrlishString();
 }
 
 /*!
@@ -662,10 +693,11 @@ static BuildInfo createBuildInfo(const Kit *k, const FilePath &projectPath,
     QmakeExtraBuildInfo extraInfo;
     BuildInfo info;
     QString suffix;
+    info.enabledByDefault = type == BuildConfiguration::Debug;
 
     if (type == BuildConfiguration::Release) {
         //: The name of the release build configuration created by default for a qmake project.
-        info.displayName = ::ProjectExplorer::Tr::tr("Release");
+        info.displayName = msgBuildConfigurationRelease();
         //: Non-ASCII characters in directory suffix may cause build issues.
         suffix = Tr::tr("Release", "Shadow build directory suffix");
         if (settings.qtQuickCompiler() == TriState::Default) {
@@ -675,12 +707,12 @@ static BuildInfo createBuildInfo(const Kit *k, const FilePath &projectPath,
     } else {
         if (type == BuildConfiguration::Debug) {
             //: The name of the debug build configuration created by default for a qmake project.
-            info.displayName = ::ProjectExplorer::Tr::tr("Debug");
+            info.displayName = msgBuildConfigurationDebug();
             //: Non-ASCII characters in directory suffix may cause build issues.
             suffix = Tr::tr("Debug", "Shadow build directory suffix");
         } else if (type == BuildConfiguration::Profile) {
             //: The name of the profile build configuration created by default for a qmake project.
-            info.displayName = ::ProjectExplorer::Tr::tr("Profile");
+            info.displayName = msgBuildConfigurationProfile();
             //: Non-ASCII characters in directory suffix may cause build issues.
             suffix = Tr::tr("Profile", "Shadow build directory suffix");
             if (settings.separateDebugInfo() == TriState::Default)
@@ -703,9 +735,9 @@ static BuildInfo createBuildInfo(const Kit *k, const FilePath &projectPath,
     if (version && version->isInQtSourceDirectory(projectPath)) {
         // assemble build directory
         QString projectDirectory = projectPath.toFileInfo().absolutePath();
-        QDir qtSourceDir = QDir(version->sourcePath().toString());
+        QDir qtSourceDir = QDir(version->sourcePath().toUrlishString());
         QString relativeProjectPath = qtSourceDir.relativeFilePath(projectDirectory);
-        QString qtBuildDir = version->prefix().toString();
+        QString qtBuildDir = version->prefix().toUrlishString();
         QString absoluteBuildPath = QDir::cleanPath(qtBuildDir + QLatin1Char('/') + relativeProjectPath);
 
         info.buildDirectory = FilePath::fromString(absoluteBuildPath);
@@ -781,7 +813,7 @@ QmakeBuildConfiguration::LastKitState::LastKitState() = default;
 
 QmakeBuildConfiguration::LastKitState::LastKitState(Kit *k)
     : m_qtVersion(QtKitAspect::qtVersionId(k)),
-      m_sysroot(SysRootKitAspect::sysRoot(k).toString()),
+      m_sysroot(SysRootKitAspect::sysRoot(k).toUrlishString()),
       m_mkspec(QmakeKitAspect::mkspec(k))
 {
     Toolchain *tc = ToolchainKitAspect::cxxToolchain(k);
