@@ -1,35 +1,7 @@
-#############################################################################
-##
-## Copyright (C) 2015 The Qt Company Ltd.
-## Contact: http://www.qt.io/licensing
-##
-## This file is part of Qt Creator.
-##
-## Commercial License Usage
-## Licensees holding valid commercial Qt licenses may use this file in
-## accordance with the commercial license agreement provided with the
-## Software or, alternatively, in accordance with the terms contained in
-## a written agreement between you and The Qt Company.  For licensing terms and
-## conditions see http://www.qt.io/terms-conditions.  For further information
-## use the contact form at http://www.qt.io/contact-us.
-##
-## GNU Lesser General Public License Usage
-## Alternatively, this file may be used under the terms of the GNU Lesser
-## General Public License version 2.1 or version 3 as published by the Free
-## Software Foundation and appearing in the file LICENSE.LGPLv21 and
-## LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-## following information to ensure the GNU Lesser General Public License
-## requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-## http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-##
-## In addition, as a special exception, The Qt Company gives you certain additional
-## rights.  These rights are described in The Qt Company LGPL Exception
-## version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
-##
-#############################################################################
+# Copyright (C) 2016 The Qt Company Ltd.
+# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 source("../../shared/qtcreator.py")
-import __builtin__
 
 cppEditorStr = ":Qt Creator_CppEditor::Internal::CPPEditorWidget"
 originalSources = os.path.abspath(os.path.join(os.getcwd(), "..", "shared", "simplePlainCPP"))
@@ -49,7 +21,7 @@ def main():
         test.fatal("Could not prepare test files - leaving test")
         return
     proFile = os.path.join(folder, "testfiles.pro")
-    startApplication("qtcreator" + SettingsPath)
+    startQC()
     if not startedWithoutPluginError():
         return
     openQmakeProject(proFile)
@@ -57,11 +29,11 @@ def main():
                          "testfiles.Headers.testfile\\.h":__modifyHeader__,
                          "testfiles.Sources.testfile\\.cpp":__modifySource__,
                          "testfiles.Sources.main\\.cpp":None}
-    for fileName, modification in fileModifications.iteritems():
+    for fileName, modification in fileModifications.items():
         __modifyFile__(fileName, modification)
     test.log("Reverting all files...")
     fileModifications = dict(zip(fileModifications.keys(),
-                                 (__builtin__.bool(v) for v in fileModifications.values())))
+                                 (builtins.bool(v) for v in fileModifications.values())))
     revertChanges(fileModifications)
     invokeMenuItem("File", "Exit")
 
@@ -112,7 +84,7 @@ def __modifySource__():
         type(cppEditorStr, "<Delete>")
 
 def revertChanges(files):
-    for f,canRevert in files.iteritems():
+    for f,canRevert in files.items():
         simpleName = simpleFileName(f)
         test.log("Trying to revert changes for '%s'" % simpleName)
         if openDocument(f):
